'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getNormalizedEntity = getNormalizedEntity;

var _shortid = require('shortid');

var _shortid2 = _interopRequireDefault(_shortid);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getNormalizedEntity(entity, config) {
  // unpack
  var patch = config.patch || {};
  var isDeleteJoin = config.isDeleteJoin,
      schema = config.schema;

  var depth = config.depth || 0;
  if (!schema) {
    console.warn('You need a schema for normalizing this entity');
  }
  var collectionName = schema.collectionName,
      items = schema.model.items,
      itemsByKey = schema.itemsByKey,
      pluralSchemasByKey = schema.pluralSchemasByKey,
      singleNamesByPluralName = schema.singleNamesByPluralName,
      singleSchemasByKey = schema.singleSchemasByKey;
  // init

  var normalizedEntity = Object.assign({}, entity);
  // parse
  Object.keys(singleSchemasByKey).forEach(function (key) {
    var item = itemsByKey[key];
    var subEntity = normalizedEntity[item.from || key];
    if (subEntity) {
      // schema collection
      var subSingleSchema = singleSchemasByKey[key];
      var subCollectionName = subSingleSchema.collectionName;
      var subCollectionKey = subCollectionName + 'ById';
      // id
      var subEntityId = subEntity.id;
      if (!subEntityId) {
        subEntityId = (0, _utils.getNewId)();
        subEntity = Object.assign({ id: subEntityId }, subEntity);
      }
      // set to normalized patch
      var patchCollection = patch[subCollectionKey];
      if (!patchCollection) {
        patch[subCollectionKey] = {};
        patchCollection = patch[subCollectionKey];
      }
      // recursive
      var subNormalizedEntity = getNormalizedEntity(subEntity, { depth: depth + 1,
        isDeleteJoin: isDeleteJoin,
        patch: patch,
        schema: subSingleSchema
      });
      // delete it
      if (isDeleteJoin) {
        delete normalizedEntity[key];
      }
      // set inside
      normalizedEntity[key + 'Id'] = subEntityId;
      // set to global patch
      if (!patchCollection[subEntityId]) {
        patchCollection[subEntityId] = subNormalizedEntity;
      }
    }
  });
  Object.keys(pluralSchemasByKey).forEach(function (key) {
    var subPluralName = key.slice(0, -4);
    var subPluralSchema = pluralSchemasByKey[key];
    var subSingleName = singleNamesByPluralName[subPluralName];
    // the subCollection could be either
    // object shape { <id1>: <entity1>, <id2>: <entity2>, ... }
    // or array shape [ <entity1>, <entity2>, ... ]
    var subEntities = void 0,
        subEntityIds = void 0;
    // check first if the from key is specified
    var from = itemsByKey[key].from;

    if (from) {
      var fromVariable = normalizedEntity[from];
      if (fromVariable) {
        if (!Array.isArray(fromVariable)) {
          // map
          subEntityIds = Object.keys(fromVariable);
          // set the internal id equal to the key if no exist
          // skip the ones that have id not equal to the key
          subEntities = subEntityIds.map(function (subEntityId) {
            var subEntity = fromVariable[subEntityId];
            if (!subEntity.id) {
              subEntity.id = subEntityId;
            } else if (subEntity.id !== subEntityId) {
              console.warn('subEntity.id !== subEntityId here for ' + subPluralName);
              return null;
            }
            return subEntity;
          }).filter(function (subEntity) {
            return subEntity;
          });
          // delete it
          if (isDeleteJoin) {
            delete normalizedEntity[from];
          }
        } else {
          subEntities = fromVariable;
          subEntityIds = fromVariable.map(function (_ref) {
            var id = _ref.id;
            return id;
          });
          // delete it
          if (isDeleteJoin) {
            delete normalizedEntity[from];
          }
        }
      }
    } else {
      var subPlural = normalizedEntity[key];
      if (subPlural) {
        // map
        subEntityIds = Object.keys(subPlural);
        // skip the ones that have id not equal to the key
        subEntities = subEntityIds.map(function (subEntityId) {
          var subEntity = subPlural[subEntityId];
          if (!subEntity.id) {
            subEntity.id = subEntityId;
          } else if (subEntity.id !== subEntityId) {
            console.warn('subEntity.id !== subEntityId here for ' + subPluralName);
          }
          return subEntity;
        });
        // delete it
        if (isDeleteJoin) {
          delete normalizedEntity[key];
        }
      } else {
        // map
        subEntities = normalizedEntity[subPluralName];
        if (subEntities) {
          // id
          subEntities = subEntities.map(function (subEntity) {
            if (!subEntity.id) {
              return Object.assign({ id: (0, _utils.getNewId)() }, subEntity);
            }
            return subEntity;
          });
          // map
          subEntityIds = subEntities.map(function (_ref2) {
            var id = _ref2.id;
            return id;
          });
          // delete it
          if (isDeleteJoin) {
            delete normalizedEntity[subPluralName];
          }
        } else {
          // console.warn(`no such ${key} neither ${subPluralName} in ${collectionName} entity`)
        }
      }
    }
    if (subEntities) {
      // set to normalized patch
      var subCollectionName = subPluralSchema.collectionName;
      var subCollectionKey = subCollectionName + 'ById';
      var patchCollection = patch[subCollectionKey];
      if (!patchCollection) {
        patch[subCollectionKey] = {};
        patchCollection = patch[subCollectionKey];
      }
      // recursive
      subEntities.forEach(function (subEntity) {
        // set the id from the key if not defined
        var subEntityId = subEntity.id;
        // even if the patchCollection[subEntityId] exists already
        // we need to parse it in order to collect potential joined entities
        var subNormalizedEntity = getNormalizedEntity(subEntity, { depth: depth + 1,
          isDeleteJoin: isDeleteJoin,
          patch: patch,
          schema: subPluralSchema
        });
        // set
        if (!patchCollection[subEntityId]) {
          patchCollection[subEntityId] = subNormalizedEntity;
        }
      });
      // set inside
      normalizedEntity[subSingleName + 'Ids'] = subEntityIds;
    }
  });
  // return
  return normalizedEntity;
}