'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getJoins = getJoins;
exports.getJoinedState = getJoinedState;
exports.getResolvedJoin = getResolvedJoin;

var _transactionsToolsIsomorphic = require('transactions-tools-isomorphic');

var _package = require('./package');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function getJoins() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  // unpack
  var isAllDeepJoins = config.isAllDeepJoins,
      joinKey = config.joinKey,
      parentJoinKey = config.parentJoinKey,
      schemasByJoinKey = config.schema.schemasByJoinKey;

  var depth = config.depth || 0;
  var maxDepth = config.maxDepth || 4;
  // check
  if (depth === maxDepth) {
    return [];
  }
  // map
  var joins = Object.keys(schemasByJoinKey).map(function (key) {
    // init
    var join = { key: key
      // deep and avoid circular
    };if (isAllDeepJoins && key !== parentJoinKey) {
      // schema
      var schema = schemasByJoinKey[key];
      if (!schema) {
        console.warn('No such schema for ' + key);
        return join;
      }
      // recursive
      join.joins = getJoins({ depth: depth + 1,
        isAllDeepJoins: isAllDeepJoins,
        joinKey: key,
        maxDepth: maxDepth,
        parentJoinKey: joinKey,
        schema: schema
      });
    }
    // return
    return join;
  });
  // return
  return joins;
}

function getJoinedState(_ref) {
  var collectionName = _ref.collectionName,
      entities = _ref.entities,
      entity = _ref.entity,
      isSingular = _ref.isSingular,
      joins = _ref.joins,
      setKey = _ref.setKey,
      shortJoinKey = _ref.shortJoinKey;
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  // unpack
  var id = config.id,
      parentEntity = config.parentEntity,
      parentEntities = config.parentEntities;

  var isNested = typeof config.isNested === 'undefined' ? true : config.isNested;
  var depth = config.depth || 0;
  var state = config.state || {};
  // prepare and copy in the nested case
  // in order to not mutate the normalizer
  var setEntities = entities && isNested ? [].concat(_toConsumableArray(entities.map(function (entityVariable) {
    return Array.isArray(entityVariable) ? entityVariable.map(function (arrayEntity) {
      return (
        // pass to the joined children the joins
        Object.assign({}, joins && { joins: joins }, arrayEntity)
      );
    }) : Object.assign({}, joins && { joins: joins }, entityVariable);
  }))) : entities;
  var setEntity = entity && isNested
  // pass to the joined children the joins
  ? Object.assign({}, joins && { joins: joins }, entity) : entity;
  /*
  console.log('setEntities', setEntities, 'setEntity', setEntity,
    'parentEntities', parentEntities, 'parentEntity', parentEntity,
  'joins', joins)
  */
  // nested case
  if (isNested && (parentEntity || parentEntities)) {
    if (setEntities) {
      if (parentEntity) {
        // no need to go deeper if it already set
        if (parentEntity[shortJoinKey]) {
          return state;
        }
        // set
        parentEntity[shortJoinKey] = setEntities;
      } else if (parentEntities) {
        parentEntities.forEach(function (parentVariable, index) {
          if (Array.isArray(parentVariable)) {
            parentVariable.forEach(function (parentEntity, entityIndex) {
              parentEntity[shortJoinKey] = setEntities[index][entityIndex];
            });
          } else {
            parentVariable[shortJoinKey] = isSingular ? setEntities[index] : Array.isArray(setEntities[index]) ? setEntities[index] : setEntities;
          }
        });
      }
    } else if (setEntity && setEntity.id !== id) {
      // avoid circular set
      if (parentEntity) {
        parentEntity[shortJoinKey] = setEntity;
      } else if (parentEntities) {
        parentEntities.forEach(function (parentEntity) {
          parentEntity[shortJoinKey] = setEntity;
        });
      }
    } else {
      return;
    }
  } else {
    if (setEntities) {
      state[setKey] = setEntities;
    } else if (setEntity && setEntity.id !== id) {
      // avoid circular set
      state[setKey] = setEntity;
    } else {
      // return { shortJoinKey }
    }
  }
  // recursive
  if (joins) {
    joins.forEach(function (join) {
      return getJoinedState(join, { depth: depth + 1,
        id: id,
        isNested: isNested,
        parentEntity: setEntity,
        parentEntities: setEntities,
        state: state
      });
    });
  }
  // return
  return state;
}

function getResolvedJoin(join) {
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var _ref2 = arguments[2];
  var state = _ref2.state;

  // unpack
  var entities = config.entities,
      entity = config.entity,
      parentSchema = config.parentSchema,
      schema = config.schema;
  var as = join.as,
      joins = join.joins,
      key = join.key,
      setKey = join.setKey;
  var singleSchemasByIdKey = schema.singleSchemasByIdKey,
      pluralNamesBySingleName = schema.pluralNamesBySingleName,
      pluralSchemasByIdsKey = schema.pluralSchemasByIdsKey;

  var path = config.path;
  // parse
  var depth = config.depth || 0;
  var isSingular = true;
  var joinSchema = singleSchemasByIdKey[key];
  if (!joinSchema) {
    joinSchema = pluralSchemasByIdsKey[key];
    isSingular = false;
  }
  // check
  if (!joinSchema) {
    console.warn('no joinSchema for ' + key + ' in ' + schema.collectionName);
    return;
  }
  // cloned resolved join
  var joinCollectionName = joinSchema.collectionName;
  var resolvedJoin = Object.assign({}, join, { collectionName: joinCollectionName,
    isSingular: isSingular,
    setKey: as
  });
  var joinConfig = { depth: depth + 1,
    schema: joinSchema,
    state: state
    // join keys
  };if (!as) {
    var joinKey = key.slice(0, -2);
    // set
    resolvedJoin.isSingular = isSingular;
    resolvedJoin.shortJoinKey = joinKey;
    // now adapt for the setKey
    var parentJoinKey = config.parentJoinKey || '';
    if (isSingular) {
      if (entity) {
        // console.log('SINGULAR entity', 'path', path, 'joinKey', joinKey, 'parentJoinKey', parentJoinKey)
      } else if (entities) {
        // console.log('SINGULAR entities', 'path', path, 'joinKey', joinKey, 'parentJoinKey', parentJoinKey)
        path = path.slice(0, -parentJoinKey.length);
        parentJoinKey = parentSchema.singleNamesByPluralName[parentJoinKey];
        path = path + parentJoinKey;
        joinKey = pluralNamesBySingleName[joinKey];
      }
    } else {
      joinKey = !as && pluralNamesBySingleName[key.slice(0, -3)];
      resolvedJoin.shortJoinKey = joinKey;
      joinSchema = pluralSchemasByIdsKey[key];
      if (entity) {
        // console.log('PLURAL entity', 'path', path, 'joinKey', joinKey, 'parentJoinKey', parentJoinKey)
        // joinKey = schema.singleNamesByPluralName[joinKey]
      } else if (entities) {
        // console.log('PLURAL entities', 'path', path, 'joinKey', joinKey, 'parentJoinKey', parentJoinKey)
      }
    }
    // set
    resolvedJoin.setKey = path ? '' + path + joinKey[0].toUpperCase() + joinKey.slice(1) : joinKey;
    Object.assign(joinConfig, {
      path: resolvedJoin.setKey,
      parentJoinKey: joinKey,
      parentSchema: schema
    });
  }
  // check
  if (isSingular) {
    if (entity) {
      // console.log('SINGULAR entity', entity)
      // const joinId = (entity || entities && entities[0] || {})[key]
      var joinId = entity[key];
      var joinEntity = joinId && (0, _package.getNormalizerEntity)({ normalizer: state }, joinCollectionName, joinId);
      // all the joins are not completed so just drop it
      if (!joinEntity) {
        // console.log('RETURN')
        return resolvedJoin;
      }
      // set
      resolvedJoin.entity = joinEntity;
      // recursive
      if (joins) {
        joinConfig.entity = joinEntity;
        resolvedJoin.joins = joins.map(function (join) {
          return getResolvedJoin(join, joinConfig, { state: state });
        });
      }
    } else if (entities) {
      var joinVariable = entities.map(function (entityVariable) {
        return Array.isArray(entityVariable) ? entityVariable.map(function (entity) {
          return entity[key];
        }) : entityVariable && entityVariable[key];
      });
      var joinEntities = joinVariable && (Array.isArray(joinVariable[0]) ? joinVariable.map(function (joinIds) {
        return joinIds && (0, _package.getNormalizerEntities)({ normalizer: state }, joinCollectionName, joinIds);
      }) : (0, _package.getNormalizerEntities)({ normalizer: state }, joinCollectionName, joinVariable));
      // check
      if (!joinEntities || joinEntities.length === 0 || joinEntities.some(function (joinEntity) {
        return !joinEntity;
      })) {
        return resolvedJoin;
      }
      // set
      resolvedJoin.entities = joinEntities;
      // recursive
      if (joins) {
        joinConfig.entities = joinEntities;
        resolvedJoin.joins = joins.map(function (join) {
          return getResolvedJoin(join, joinConfig, { state: state });
        });
      }
    }
  } else {
    if (entity) {
      // console.log('PLURAL entity', entity, 'entities', entities)
      var joinIds = entity && entity[key];
      var _joinEntities = joinIds && (0, _package.getNormalizerEntities)({ normalizer: state }, joinCollectionName, joinIds);
      // console.log('joinEntities', joinEntities)
      // check
      if (!_joinEntities || _joinEntities.length === 0 || _joinEntities.some(function (joinEntity) {
        return !joinEntity;
      })) {
        // console.log('RETURN')
        return resolvedJoin;
      }
      // set
      resolvedJoin.entities = _joinEntities;
      // recursive
      if (joins) {
        joinConfig.entities = _joinEntities;
        resolvedJoin.joins = joins.map(function (join) {
          return getResolvedJoin(join, joinConfig, { state: state });
        });
      }
    } else if (entities) {
      // console.log('PLURAL entities', entities)
      var joinIdsArrays = entities.map(function (entity) {
        return entity && entity[key] || [];
      });
      var joinEntitiesArrays = joinIdsArrays.map(function (joinIds) {
        return joinIds && (0, _package.getNormalizerEntities)({ normalizer: state }, joinCollectionName, joinIds);
      });
      // check
      if (!joinEntitiesArrays || joinEntitiesArrays.every(function (joinEntities) {
        return !joinEntities || joinEntities.length === 0;
      })) {
        // console.log('RETURN')
        return resolvedJoin;
      }
      // set
      resolvedJoin.entities = joinEntitiesArrays;
      // recursive
      if (joins) {
        joinConfig.entities = joinEntitiesArrays;
        resolvedJoin.joins = joins.map(function (join) {
          return getResolvedJoin(join, joinConfig, { state: state });
        });
      }
    }
  }
  // return
  return resolvedJoin;
}