'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getNormalizedPatch = getNormalizedPatch;

var _entity = require('./entity');

function getNormalizedPatch(patch, config) {
  // unpack
  var normalizedPatch = {};
  var appSchema = config.appSchema,
      isDeleteJoin = config.isDeleteJoin;

  if (!appSchema) {
    console.warn('You need an appSchema to getNormalizedPatch');
    return patch;
  }
  var pluralSchemasByKey = appSchema.pluralSchemasByKey,
      singleSchemasByKey = appSchema.singleSchemasByKey;
  // recursive

  Object.keys(patch).forEach(function (singleOrPluralKey) {
    var singleOrPlural = patch[singleOrPluralKey];
    var singleSchema = singleSchemasByKey[singleOrPluralKey];
    if (singleSchema) {
      // normalize
      var normalizedEntity = (0, _entity.getNormalizedEntity)(singleOrPlural, { isDeleteJoin: isDeleteJoin,
        patch: normalizedPatch,
        schema: singleSchema
      });
      // set
      normalizedPatch[singleOrPluralKey] = normalizedEntity;
    } else {
      var pluralSchema = pluralSchemasByKey[singleOrPluralKey];
      var patchCollection = normalizedPatch[singleOrPluralKey];
      if (!patchCollection) {
        normalizedPatch[singleOrPluralKey] = {};
        patchCollection = normalizedPatch[singleOrPluralKey];
      }
      if (pluralSchema) {
        Object.keys(singleOrPlural).forEach(function (entityId) {
          var entity = singleOrPlural[entityId];
          // normalize
          var normalizedEntity = (0, _entity.getNormalizedEntity)(entity, { isDeleteJoin: isDeleteJoin,
            patch: normalizedPatch,
            schema: pluralSchema
          });
          // set
          patchCollection[entityId] = normalizedEntity;
        });
      }
    }
  });
  // return
  return normalizedPatch;
}