'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createReducer = createReducer;

var _transactionsReduxPackage = require('transactions-redux-package');

var _transactionsToolsIsomorphic = require('transactions-tools-isomorphic');

var _appSchema2 = require('./appSchema');

var _join = require('./join');

var _package = require('./package');

var _patch = require('./patch');

var completeTest = /COMPLETE_NORMALIZER(.*)/;

function createReducer() {
  var description = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var config = arguments[1];

  // unpack
  config = Object.assign({ methodsByName: {},
    reducer: function reducer(state, action) {
      return state;
    },
    initialState: {},
    isDeleteJoin: true,
    reducerName: 'normalizer',
    schema: null
  }, config);
  var _config = config,
      definition = _config.definition,
      methodsByName = _config.methodsByName,
      isDeleteJoin = _config.isDeleteJoin,
      reducerName = _config.reducerName;
  // update methodsByName

  if (!methodsByName.getResolvedJoin) {
    methodsByName.getResolvedJoin = _join.getResolvedJoin;
  }
  methodsByName.getState = function (config) {
    return config.state;
  };
  // appSchema
  var appSchema = config.appSchema;
  if (!appSchema) {
    if (definition) {
      appSchema = new _transactionsToolsIsomorphic.SingleSchema('app');
      appSchema.define(definition);
    } else if (description) {
      appSchema = (0, _appSchema2.createAppSchema)(description);
    } else {
      console.error('You need a description in order to build you app Schema in normalizer');
      return function (state) {
        return state;
      };
    }
  }
  // default initial state
  var _appSchema = appSchema,
      pluralSchemasByKey = _appSchema.pluralSchemasByKey,
      singleSchemasByKey = _appSchema.singleSchemasByKey;

  if (!singleSchemasByKey) {
    console.warn('Your app schema has no singleSchemasByKey');
  } else if (!pluralSchemasByKey) {
    console.warn('Your app schema has no pluralSchemasByKey');
  }
  var defaultsByCollectionName = {};
  // Simpler is to already have an initial state
  // with an empty singlesByName and empty collections
  // in which we have empty elementsById object
  var defaultInitialState = {
    getCollectionNames: function getCollectionNames() {
      return description.collectionNames;
    },
    getDefault: function getDefault(collectionName) {
      return defaultsByCollectionName[collectionName];
    },
    getSchema: function getSchema(collectionName) {
      return appSchema.pluralSchemasByKey[collectionName + 'ById'];
    }
  };
  Object.keys(singleSchemasByKey || {}).forEach(function (key) {
    defaultInitialState[key] = singleSchemasByKey[key].getDefaults();
  });
  Object.keys(appSchema.pluralSchemasByKey || {}).forEach(function (key) {
    defaultInitialState[key] = {};
    var pluralSchema = pluralSchemasByKey[key];
    defaultsByCollectionName[pluralSchema.collectionName] = pluralSchema.getDefaults();
  });
  // now feed it with custom initial state
  var normalizeConfig = { appSchema: appSchema,
    isDeleteJoin: isDeleteJoin
    // check just the good shape of the initial state
  };var initialState = (0, _transactionsReduxPackage.getMergedState)(defaultInitialState, (0, _patch.getNormalizedPatch)(config.initialState, normalizeConfig));
  // bind
  (0, _transactionsReduxPackage.setBoundState)(initialState, methodsByName, { appSchema: appSchema });
  // reducer
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments[1];

    // init
    var newState = state;
    var normalizerAction = action[reducerName];
    // check
    if (normalizerAction) {
      // new patch
      var getNewPatch = function getNewPatch(patch) {
        return (0, _patch.getNormalizedPatch)(patch, normalizeConfig);
      };
      // normalize
      newState = (0, _package.normalizer)(state, action, { getCompletedPatch: getNewPatch,
        getMergedPatch: getNewPatch
      });
      // bind
      (0, _transactionsReduxPackage.setBoundState)(newState, methodsByName, { appSchema: appSchema });
    }
    // possible heritance
    var output = config.reducer(newState, action);
    return output;
  };
}