'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.joinerReselector = exports.getJoinedStateReselector = undefined;

var _reselect = require('reselect');

var _transactionsToolsIsomorphic = require('transactions-tools-isomorphic');

var _join = require('../join');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var getJoinedStateReselector = exports.getJoinedStateReselector = function getJoinedStateReselector(join, schema) {
  var resolvedJoinReselector = (0, _reselect.createSelector)(function (state, ownProps) {
    return ownProps[join.key];
  }, function (state) {
    return state.normalizer.getResolvedJoin;
  }, function (value, getResolvedJoin) {
    return getResolvedJoin(join, {
      entity: _defineProperty({}, join.key, value),
      schema: schema
    });
  });
  var resolvedVariableReselector = (0, _reselect.createSelector)(resolvedJoinReselector, function (state, ownProps) {
    return ownProps;
  }, function (_ref, ownProps) {
    var shortJoinKey = _ref.shortJoinKey;
    return ownProps[shortJoinKey];
  });
  return (0, _reselect.createSelector)(resolvedJoinReselector, resolvedVariableReselector, function (state, ownProps) {
    return ownProps.id;
  }, function (resolvedJoin, resolvedVariable, id) {
    return !resolvedVariable && (0, _join.getJoinedState)(resolvedJoin, { id: id, isNested: true });
  });
};

var getJoiner = function getJoiner(_ref2, getSchema) {
  var collectionName = _ref2.collectionName,
      query = _ref2.query;

  // schema
  var schema = getSchema(collectionName);
  if (!schema) {
    console.warn('No such schema for ' + collectionName);
  }
  // init
  var joins = query && query[_transactionsToolsIsomorphic.JOINS];
  // determine joins
  var isAllJoins = query && query[_transactionsToolsIsomorphic.IS_ALL_JOINS];
  if (!joins && isAllJoins) {
    joins = (0, _join.getJoins)({ schema: schema,
      isAllDeepJoins: query[_transactionsToolsIsomorphic.IS_ALL_DEEP_JOINS],
      maxDepth: query[_transactionsToolsIsomorphic.MAX_DEPTH]
    });
  }
  // warning
  if (!joins) {
    // console.warn(`You forgot to define some joins for a ${collectionName} Component`)
    return {};
  }
  // create
  var joinedStateReselectors = joins.map(function (join) {
    return getJoinedStateReselector(join, schema);
  });
  // return
  return { joinedStateReselectors: joinedStateReselectors };
};

var joinerReselector = exports.joinerReselector = (0, _reselect.createSelector)(function (option) {
  return option;
}, function (option, getSchema) {
  return getSchema;
}, getJoiner);