import test from 'ava'
const { createAppSchema,
  getNormalizedPatch
} = require('../lib/utils')

const description = {
  article: {
    items: [
      { key: 'id', type: 'String' },
      { key: 'linkId', type: 'String' }
    ],
    title: 'article',
    type: 'collection'
  },
  author: {
    items: [
      { key: 'id', type: 'String', default: null, isAutomatic: true },
      { key: 'name', type: 'String', default: null }
    ],
    title: 'author',
    type: 'collection'
  },
  link: {
    items: [
      { key: 'authorId', type: 'String', isAutomatic: true },
      { key: 'id', type: 'String', isAutomatic: true }
    ],
    title: 'link',
    type: 'collection'
  }
}

const theArticle = { id: '1', link: { id: '2', author: { id: '3', name: 'saintloup'}}}
const theApp = { articlesById: { 1: theArticle } }

test('createAppSchema', t => {
  const appSchema = createAppSchema(description)
  t.pass()
})

test('getNormalizedPatch', t => {
  const normalizedPatch = getNormalizedPatch(theApp, appSchema)
  t.deepEqual([
    normalizedPatch.articlesById['1'].linkId,
    normalizedPatch.linksById['2'].authorId
  ], [
    '2',
    '3'
  ])
})
