import test from 'ava'

import { _DELETE_,
  createReducer,
  createAppSchema,
  getDeletedPathsWithPatch,
  mergeInState
} from '../lib'
import { descripion,
  initTestStore,
  mergedTestFoosById,
  mergedTestFaaIds,
  mergedTestFooIds
} from '../lib/tests'

test('getDeletedPathsWithPatch', t => {
  const deletePaths = getDeletedPathsWithPatch({
    '0': {
      '01': _DELETE_,
      '02': {
        '020': _DELETE_
      }
    },
    '1': {
      '10': {
        '100': _DELETE_
      },
      '11': _DELETE_
    }
  })
  t.is(deletePaths.toString(), [ [ '0', '01' ],
  [ '0', '02', '020' ],
  [ '1', '10', '100' ],
  [ '1', '11' ] ].toString())
})

test('mergeInState', t => {
  const mergedState = mergeInState({
    '0': {
      '01': _DELETE_,
      '02': {
        '020': _DELETE_
      }
    },
    '1': {
      '10': {
        '100': _DELETE_
      },
      '11': _DELETE_
    }
  })
  t.pass()
})

test('createAppSchema', t => {
  const appSchema = createAppSchema(descripion)
  t.deepEqual(
    [
      Object.keys(appSchema.foosById._itemSchema.singleSchemasByKey),
      Object.keys(appSchema.foosById._itemSchema.pluralSchemasByKey)
    ], [
      ['faa', 'myFaa', 'yourFaa'],
      ['theFaasById']
    ]
  )
})

test('createReducer', t => {
  const { store } = initTestStore()
  store.dispatch({
    type: 'MERGE_FOOS_normalizer',
    normalizer: { patch: { foosById: mergedTestFoosById } }
  })
  const state = store.getState()
  t.deepEqual([
    Object.keys(state.normalizer.foosById).sort(),
    Object.keys(state.normalizer.faasById).sort()
  ],[
    mergedTestFooIds,
    mergedTestFaaIds
  ])
})
